@@ -43,7 +43,7 @@ ShellHandler::ShellHandler(RootSurfaceContainer *rootContainer)
     , m_workspace(new Workspace(rootContainer))
     , m_topContainer(new LayerSurfaceContainer(rootContainer))
     , m_overlayContainer(new LayerSurfaceContainer(rootContainer))
-    , m_popupContainer(new PopupSurfaceContainer(rootContainer))
+    , m_popupContainer(new SurfaceContainer(rootContainer))
 {
     m_backgroundContainer->setZ(RootSurfaceContainer::BackgroundZOrder);
     m_bottomContainer->setZ(RootSurfaceContainer::BottomZOrder);
@@ -124,7 +124,10 @@ WXWayland *ShellHandler::createXWayland(WServer *server,
 void ShellHandler::removeXWayland(WXWayland *xwayland)
 {
     m_xwaylands.removeOne(xwayland);
-    xwayland->safeDeleteLater();
+    // Ensure xwayland is removed from interfaceList
+    if (auto server = WServer::from(xwayland))
+        server->detach(xwayland);
+    delete xwayland;
 }
 
 void ShellHandler::initInputMethodHelper(WServer *server, WSeat *seat)
