@@ -98,12 +98,14 @@ private:
     WAYLIB_SERVER_NAMESPACE::WInputMethodHelper *m_inputMethodHelper = nullptr;
     QList<WAYLIB_SERVER_NAMESPACE::WXWayland *> m_xwaylands;
 
-    RootSurfaceContainer *m_rootSurfaceContainer = nullptr;
+    QPointer<RootSurfaceContainer> m_rootSurfaceContainer;
     LayerSurfaceContainer *m_backgroundContainer = nullptr;
     LayerSurfaceContainer *m_bottomContainer = nullptr;
     Workspace *m_workspace = nullptr;
     LayerSurfaceContainer *m_topContainer = nullptr;
     LayerSurfaceContainer *m_overlayContainer = nullptr;
-    PopupSurfaceContainer *m_popupContainer = nullptr;
+    // FIXME: https://github.com/linuxdeepin/treeland/pull/428 Caused damage to the tooltip
+    // Need to find a better way to handle popup click events
+    SurfaceContainer *m_popupContainer = nullptr;
     QObject *m_windowMenu = nullptr;
 };
