@@ -88,6 +88,7 @@ public:
     bool canHibernate{ false };
     bool canHybridSleep{ false };
     bool isLocked{ false };
+    bool isLoggedIn{ false };
 };
 
 GreeterProxy::GreeterProxy(QObject *parent)
@@ -109,6 +110,11 @@ GreeterProxy::GreeterProxy(QObject *parent)
     connect(d->socket, &QLocalSocket::readyRead, this, &GreeterProxy::readyRead);
     connect(d->socket, &QLocalSocket::errorOccurred, this, &GreeterProxy::error);
 
+    connect(this, &GreeterProxy::loginSucceeded, this, [this]([[maybe_unused]] QString user) {
+        d->isLoggedIn = true;
+        Q_EMIT isLoggedInChanged();
+    });
+
     d->ddmDisplayManager = new org::deepin::DisplayManager("org.deepin.DisplayManager",
                                                            "/org/deepin/DisplayManager",
                                                            QDBusConnection::systemBus());
@@ -157,6 +163,11 @@ bool GreeterProxy::isLocked() const
     return d->isLocked;
 }
 
+bool GreeterProxy::isLoggedIn() const
+{
+    return d->isLoggedIn;
+}
+
 bool GreeterProxy::canPowerOff() const
 {
     return d->canPowerOff;
@@ -257,10 +268,10 @@ void GreeterProxy::login(const QString &user, const QString &password, const int
     QModelIndex index = d->sessionModel->index(sessionIndex, 0);
 
     // send command to the daemon
-    Session::Type type =
-        static_cast<Session::Type>(d->sessionModel->data(index, SessionModel::TypeRole).toInt());
+    DDM::Session::Type type =
+        static_cast<DDM::Session::Type>(d->sessionModel->data(index, SessionModel::TypeRole).toInt());
     QString name = d->sessionModel->data(index, SessionModel::FileRole).toString();
-    Session session(type, name);
+    DDM::Session session(type, name);
     SocketWriter(d->socket) << quint32(GreeterMessages::Login) << user << password << session;
 }
 
@@ -282,80 +293,14 @@ void GreeterProxy::unlock(const QString &user, const QString &password)
     }
 }
 
-static QString getSessionPathByUser(UserPtr userInfo);
-
 void GreeterProxy::logout()
 {
     qCDebug(treelandGreeter) << "Logout.";
-    auto user = userModel()->currentUser();
-    QThreadPool::globalInstance()->start([user]() {
-        const auto path = getSessionPathByUser(user);
-        if (path.isEmpty()) {
-            qCWarning(treelandGreeter, "No session logged in.");
-            return;
-        }
-        qCDebug(treelandGreeter) << "Terminate the session" << path;
-        auto reply = DDBusSender::system()
-            .service("org.freedesktop.login1")
-            .path(path)
-            .interface("org.freedesktop.login1.Session")
-            .method("Terminate")
-            .call();
-        if (reply.isError()) {
-            qCWarning(treelandGreeter) << "Failed to logout, error:" << reply.error().message();
-        }
-    });
-}
-
-QString getSessionPathByUser(UserPtr userInfo)
-{
-    if (!userInfo) {
-        qCWarning(treelandGreeter) << "No user logged in.";
-        return {};
-    }
-
-    QDBusPendingReply<QList<SessionInfo>> sessionsRelpy =
-        DDBusSender::system()
-            .service("org.freedesktop.login1")
-            .path("/org/freedesktop/login1")
-            .interface("org.freedesktop.login1.Manager")
-            .method("ListSessions")
-            .call();
-    if (sessionsRelpy.isError()) {
-        qCWarning(treelandGreeter) << "Failed to logout, error:" << sessionsRelpy.error().message();
-        return {};
-    }
-
-    QStringList userSessions;
-    const auto sessions = sessionsRelpy.value();
-    for (auto item : sessions) {
-        if (item.uid != userInfo->UID())
-            continue;
-        // TODO multiple seats.
-        if (item.seat.isEmpty())
-            continue;
-        userSessions << item.path.path();
-    }
-    std::sort(userSessions.begin(), userSessions.end(), [](const QString &s1, const QString &s2) {
-        return s1.localeAwareCompare(s2) > 0;
-    });
-    for (auto item : userSessions) {
-        QDBusPendingReply<QVariant> relpy = DDBusSender::system()
-                                                .service("org.freedesktop.login1")
-                                                .path(item)
-                                                .interface("org.freedesktop.login1.Session")
-                                                .property("Active")
-                                                .get();
-        if (relpy.value().toBool())
-            return item;
-    }
-    return {};
-}
-
-void GreeterProxy::activateUser(const QString &user)
-{
-    auto userInfo = userModel()->get(user);
-    SocketWriter(d->socket) << quint32(GreeterMessages::ActivateUser) << user;
+    d->isLoggedIn = false;
+    Q_EMIT isLoggedInChanged();
+    auto session = Helper::instance()->activeSession().lock();
+    SocketWriter(d->socket) << quint32(GreeterMessages::Logout) << session->id;
+    Helper::instance()->removeSession(session);
 }
 
 void GreeterProxy::connected()
@@ -470,7 +415,8 @@ void GreeterProxy::readyRead()
         } break;
         case DaemonMessages::UserActivateMessage: {
             QString user;
-            input >> user;
+            int sessionId;
+            input >> user >> sessionId;
 
             // NOTE: maybe DDM will active dde user.
             if (!d->userModel->getUser(user)) {
@@ -481,8 +427,10 @@ void GreeterProxy::readyRead()
             }
 
             d->userModel->setCurrentUserName(user);
+            // userLoggedIn signal is connected with Helper::updateActiveUserSession
+            Q_EMIT d->userModel->userLoggedIn(user, sessionId);
 
-            qCInfo(treelandGreeter) << "activate successfully: " << user;
+            qCInfo(treelandGreeter) << "activate successfully: " << user << ", XDG_SESSION_ID: " << sessionId;
         } break;
         case DaemonMessages::UserLoggedIn: {
             QString user;
