@@ -1,19 +1,23 @@
-// Copyright (C) 2024 JiDe Zhang <zhangjide@deepin.org>.
+// Copyright (C) 2024-2025 UnionTech Software Technology Co., Ltd.
 // SPDX-License-Identifier: Apache-2.0 OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "output.h"
 
-#include "config/treelandconfig.h"
+#include "modules/output-manager/impl/output_manager_impl.h"
+#include "outputconfig.hpp"
+#include "treelandconfig.hpp"
 #include "core/rootsurfacecontainer.h"
 #include "seat/helper.h"
 #include "surface/surfacewrapper.h"
 #include "workspace/workspace.h"
 #include "cmdline.h"
+#include "common/treelandlogging.h"
 
 #include <wcursor.h>
 #include <winputpopupsurface.h>
 #include <wlayersurface.h>
 #include <woutputitem.h>
+#include <woutputhelper.h>
 #include <woutputlayout.h>
 #include <woutputrenderwindow.h>
 #include <wquicktextureproxy.h>
@@ -30,12 +34,10 @@
 #define DIFF_APP_OFFSET_FACTOR 2.0
 #define POPUP_EDGE_MARGIN 10
 
-Q_LOGGING_CATEGORY(qLcOutput, "treeland.output")
-
 Output *Output::create(WOutput *output, QQmlEngine *engine, QObject *parent)
 {
     auto isSoftwareCursor = [](WOutput *output) -> bool {
-        return output->handle()->is_x11() || TreelandConfig::ref().forceSoftwareCursor();
+        return output->handle()->is_x11() || Helper::instance()->config()->forceSoftwareCursor();
     };
     QQmlComponent delegate(engine, "Treeland", "PrimaryOutput");
     QObject *obj = delegate.beginCreate(engine->rootContext());
@@ -46,12 +48,12 @@ Output *Output::create(WOutput *output, QQmlEngine *engine, QObject *parent)
     QQmlEngine::setObjectOwnership(outputItem, QQmlEngine::CppOwnership);
     outputItem->setOutput(output);
 
-    connect(&TreelandConfig::ref(),
+    connect(Helper::instance()->config(),
             &TreelandConfig::forceSoftwareCursorChanged,
             obj,
             [obj, output, isSoftwareCursor]() {
                 auto forceSoftwareCursor = isSoftwareCursor(output);
-                qCInfo(qLcOutput) << "forceSoftwareCursor changed to" << forceSoftwareCursor;
+                qCInfo(treelandOutput) << "forceSoftwareCursor changed to" << forceSoftwareCursor;
                 obj->setProperty("forceSoftwareCursor", forceSoftwareCursor);
             });
 
@@ -83,7 +85,10 @@ Output *Output::create(WOutput *output, QQmlEngine *engine, QObject *parent)
     // o->m_taskBar = Helper::instance()->qmlEngine()->createTaskBar(o,
     // contentItem); o->m_taskBar->setZ(RootSurfaceContainer::TaskBarZOrder);
 
-    if (!CmdLine::ref().disableDebugView()) {
+    // reset output color to config value
+    o->setOutputColor(-1, 0);
+
+    if (CmdLine::ref().enableDebugView()) {
         o->m_debugMenuBar = Helper::instance()->qmlEngine()->createMenuBar(outputItem, contentItem);
         o->m_debugMenuBar->setZ(RootSurfaceContainer::MenuBarZOrder);
 #ifndef QT_DEBUG
@@ -137,8 +142,14 @@ Output::Output(WOutputItem *output, QObject *parent)
     : SurfaceListModel(parent)
     , m_item(output)
     , minimizedSurfaces(new SurfaceFilterModel(this))
+    , m_backlight(Backlight::createForOutput(output->output()))
 {
     m_outputViewport = output->property("screenViewport").value<WOutputViewport *>();
+
+    // TODO: Investigate better ways to track the panel specific persistent settings.
+    // The connector name of the panel may change.
+    QString outputName = output->output()->name();
+    m_config = OutputConfig::createByName("org.deepin.treeland.outputs", "org.deepin.treeland", "/" + outputName, this);
 }
 
 Output::~Output()
@@ -238,7 +249,7 @@ void Output::placeSmartCascaded(SurfaceWrapper *surface)
         ? DIFF_APP_OFFSET_FACTOR
         : SAME_APP_OFFSET_FACTOR;
     const QRectF titleBarGeometry = latestActiveSurface->titlebarGeometry();
-    qreal offset = (titleBarGeometry.isNull() ? TreelandConfig::ref().windowTitlebarHeight()
+    qreal offset = (titleBarGeometry.isNull() ? Helper::instance()->config()->windowTitlebarHeight()
                                               : titleBarGeometry.height())
         * factor;
 
@@ -349,7 +360,7 @@ void Output::enable()
     // Don't care for WOutput::isEnabled, must do WOutput::commit here,
     // In order to ensure trigger QWOutput::frame signal, WOutputRenderWindow
     // needs this signal to render next frame. Because QWOutput::frame signal
-    // maybe emit before WOutputRenderWindow::attach, if no commit here,
+    // maybe Q_EMIT before WOutputRenderWindow::attach, if no commit here,
     // WOutputRenderWindow will ignore this output on render.
     if (!qwoutput->property("_Enabled").toBool()) {
         qwoutput->setProperty("_Enabled", true);
@@ -432,6 +443,7 @@ void Output::addSurface(SurfaceWrapper *surface)
 void Output::removeSurface(SurfaceWrapper *surface)
 {
     clearPopupCache(surface);
+    m_initialWindowPositionRatio.remove(surface);
     Q_ASSERT(hasSurface(surface));
     SurfaceListModel::removeSurface(surface);
     surface->disconnect(this);
@@ -578,7 +590,7 @@ void Output::arrangeLayerSurface(SurfaceWrapper *surface)
             setExclusiveZone(Qt::RightEdge, layer, layer->exclusiveZone());
             break;
         default:
-            qCWarning(qLcOutput) << layer->appId()
+            qCWarning(treelandOutput) << layer->appId()
                                  << " has set exclusive zone, but exclusive edge is invalid!";
             break;
         }
@@ -612,6 +624,9 @@ void Output::arrangeLayerSurfaces()
 
 void Output::arrangeNonLayerSurface(SurfaceWrapper *surface, const QSizeF &sizeDiff)
 {
+    if (surface->type() == SurfaceWrapper::Type::LockScreen) {
+        return;
+    }
     Q_ASSERT(surface->type() != SurfaceWrapper::Type::Layer);
     surface->setFullscreenGeometry(geometry());
     const auto validGeo = this->validGeometry();
@@ -653,12 +668,37 @@ void Output::arrangeNonLayerSurface(SurfaceWrapper *surface, const QSizeF &sizeD
                 }
             }
         } else if (!sizeDiff.isNull() && sizeDiff.isValid()) {
-            const QSizeF outputSize = m_item->size();
-            const auto xScale = outputSize.width() / (outputSize.width() - sizeDiff.width());
-            const auto yScale = outputSize.height() / (outputSize.height() - sizeDiff.height());
-            normalGeo.moveLeft(normalGeo.x() * xScale);
-            normalGeo.moveTop(normalGeo.y() * yScale);
-            surface->moveNormalGeometryInOutput(normalGeo.topLeft());
+            QRectF validGeo = this->validGeometry();
+            // Save the window's proportional position relative to the available area during the initial scale.
+            if (!m_initialWindowPositionRatio.contains(surface)) {
+                qreal xRatio = 0.5, yRatio = 0.5; // Default center position
+                if (validGeo.width() > normalGeo.width()) {
+                    xRatio = (normalGeo.x() - validGeo.x()) / (validGeo.width() - normalGeo.width());
+                    xRatio = qBound(0.0, xRatio, 1.0);
+                }
+                if (validGeo.height() > normalGeo.height()) {
+                    yRatio = (normalGeo.y() - validGeo.y()) / (validGeo.height() - normalGeo.height());
+                    yRatio = qBound(0.0, yRatio, 1.0);
+                }
+                m_initialWindowPositionRatio[surface] = QPointF(xRatio, yRatio);
+            }
+
+            QPointF ratio = m_initialWindowPositionRatio[surface];
+            QPointF newPos;
+            newPos.setX(validGeo.x() + ratio.x() * (validGeo.width() - normalGeo.width()));
+            newPos.setY(validGeo.y() + ratio.y() * (validGeo.height() - normalGeo.height()));
+
+            // Boundary protection ensures that at least 30% of the window remains within the screen.
+            const qreal minVisibleRatio = 0.3;
+            const int minVisibleX = qMin(100, int(normalGeo.width() * minVisibleRatio));
+            const int minVisibleY = qMin(100, int(normalGeo.height() * minVisibleRatio));
+            newPos.setX(qBound(validGeo.left() - normalGeo.width() + minVisibleX,
+                            newPos.x(),
+                            validGeo.right() - minVisibleX));
+            newPos.setY(qBound(validGeo.top() - normalGeo.height() + minVisibleY,
+                            newPos.y(),
+                            validGeo.bottom() - minVisibleY));
+            surface->moveNormalGeometryInOutput(newPos);
         } else {
             QPoint clientRequstPos = surface->clientRequstPos();
             if (!clientRequstPos.isNull()) {
@@ -674,7 +714,7 @@ QPointF Output::calculateBasePosition(SurfaceWrapper *surface, const QPointF &dP
 {
     auto parent = surface->parentSurface();
     if (!parent || !parent->surfaceItem()) {
-        qCWarning(qLcOutput) << " Invalid parent surface or surface item!";
+        qCWarning(treelandOutput) << " Invalid parent surface or surface item!";
         return QPointF();
     }
 
@@ -702,7 +742,7 @@ void Output::adjustToOutputBounds(QPointF &pos, const QRectF &normalGeo, const Q
 void Output::handleLayerShellPopup(SurfaceWrapper *surface, const QRectF &normalGeo)
 {
     if (!surface->parentSurface() || !surface->parentSurface()->ownsOutput()) {
-        qCWarning(qLcOutput) << " Invalid LayerShell parent surface!";
+        qCWarning(treelandOutput) << " Invalid LayerShell parent surface!";
         return;
     }
 
@@ -711,7 +751,7 @@ void Output::handleLayerShellPopup(SurfaceWrapper *surface, const QRectF &normal
     auto inputPopupSurface = qobject_cast<WInputPopupSurface *>(surface->shellSurface());
 
     if (!xdgPopupSurfaceItem && !inputPopupSurface) {
-        qCWarning(qLcOutput) << " Invalid popup surface type!";
+        qCWarning(treelandOutput) << " Invalid popup surface type!";
         return;
     }
 
@@ -740,7 +780,7 @@ void Output::handleRegularPopup(SurfaceWrapper *surface, const QRectF &normalGeo
     auto inputPopupSurface = qobject_cast<WInputPopupSurface *>(surface->shellSurface());
 
     if (!xdgPopupSurfaceItem && !inputPopupSurface) {
-        qCWarning(qLcOutput) << " Invalid popup surface type!";
+        qCWarning(treelandOutput) << " Invalid popup surface type!";
         return;
     }
 
@@ -858,3 +898,130 @@ QRectF Output::validGeometry() const
 {
     return geometry().marginsRemoved(m_exclusiveZone);
 }
+
+// do not use config()->setBrightness or config()->setColorTemperature to set color temperature or brightness
+// as doing so will have no effect.
+// use Output::setOutputColor instead
+OutputConfig *Output::config() const
+{
+    return m_config;
+}
+
+namespace {
+static inline void kelvinToRGB(double kelvin, double &r, double &g, double &b)
+{
+    kelvin = std::clamp(kelvin, 1000.0, 20000.0) / 100.0;
+
+    if (kelvin <= 66.0) r = 1.0;
+    else r = std::clamp(329.698727446 * std::pow(kelvin - 60.0, -0.1332047592) / 255.0, 0.0, 1.0);
+
+    if (kelvin <= 66.0)
+        g = std::clamp((99.4708025861 * std::log(kelvin) - 161.1195681661) / 255.0, 0.0, 1.0);
+    else
+        g = std::clamp(288.1221695283 * std::pow(kelvin - 60.0, -0.0755148492) / 255.0, 0.0, 1.0);
+
+    if (kelvin >= 66.0) b = 1.0;
+    else if (kelvin <= 19.0) b = 0.0;
+    else b = std::clamp((138.5177312231 * std::log(kelvin - 10.0) - 305.0447927307) / 255.0, 0.0, 1.0);
+}
+
+static inline void generateGammaLUT(uint32_t colorTemperature,
+                                    qreal brightness,
+                                    size_t gammaSize,
+                                    QVector<uint16_t> &r,
+                                    QVector<uint16_t> &g,
+                                    QVector<uint16_t> &b)
+{
+    if (gammaSize == 0) {
+        return;
+    }
+
+    double cr, cg, cb;
+    kelvinToRGB(static_cast<double>(colorTemperature), cr, cg, cb);
+    for (size_t i = 0; i < gammaSize; ++i) {
+        double normalized = static_cast<double>(i) / static_cast<double>(gammaSize - 1);
+
+        double rValue = std::clamp(normalized * cr * brightness, 0.0, 1.0);
+        double gValue = std::clamp(normalized * cg * brightness, 0.0, 1.0);
+        double bValue = std::clamp(normalized * cb * brightness, 0.0, 1.0);
+        r[i] = static_cast<uint16_t>(std::round(rValue * 65535.0));
+        g[i] = static_cast<uint16_t>(std::round(gValue * 65535.0));
+        b[i] = static_cast<uint16_t>(std::round(bValue * 65535.0));
+    }
+}
+
+}
+
+// TODO: better Chromatic Adaptation algorithms can be implemented when the wlr_color_transform
+// api is available. For now RGB scaling is used due to limitation of gamma LUT table.
+// see: http://www.brucelindbloom.com/index.html?ChromAdaptEval.html
+void Output::setOutputColor(qreal brightness, uint32_t colorTemperature)
+{
+    if (brightness < 0)
+        brightness = config()->brightness();
+    if (colorTemperature == 0)
+        colorTemperature = config()->colorTemperature();
+
+    qreal brightnessCorrection = 1.0;
+
+    if (m_backlight) {
+        qreal backlightBrightness = m_backlight->setBrightness(brightness);
+        if (backlightBrightness != 0)
+            brightnessCorrection = qBound(0.0, brightness / backlightBrightness, 1.0);
+    } else {
+        brightnessCorrection = brightness;
+    }
+
+    const size_t gammaSize = output()->handle()->get_gamma_size();
+
+    auto *sender = qobject_cast<treeland_output_color_control_v1 *>(QObject::sender());
+
+    if (gammaSize == 0) {
+        if (sender)
+            sender->sendCommitResult(false);
+        qCWarning(treelandOutput) << " Output " << output()->name()
+                             << " does not support gamma LUT! Brightness and color temperature adjustments through gamma will have no effect.";
+        return;
+    }
+
+    QVector<uint16_t> r(gammaSize);
+    QVector<uint16_t> g(gammaSize);
+    QVector<uint16_t> b(gammaSize);
+
+    generateGammaLUT(colorTemperature,
+                     brightnessCorrection,
+                     gammaSize,
+                     r,
+                     g,
+                     b);
+    
+    WOutputHelper::ExtraState newState;
+    auto *viewport = screenViewport();
+    auto *renderWindow = screenViewport()->outputRenderWindow();
+    wlr_output_state_set_gamma_lut(newState.get(), gammaSize,
+                                   r.constData(),
+                                   g.constData(),
+                                   b.constData());
+    
+    auto *outputHelper = renderWindow->getOutputHelper(viewport);
+    outputHelper->setExtraState(newState);
+
+    outputHelper->scheduleCommitJob([this, brightness, colorTemperature, newState, sender](bool success, WOutputHelper::ExtraState state) {
+        if (state == newState) {
+            if (sender)
+                sender->sendCommitResult(success);
+            if (!success) {
+                qCWarning(treelandOutput) << "Failed to apply brightness and color temperature settings to output"
+                                          << output()->name();
+            } else {
+                config()->setBrightness(brightness);
+                config()->setColorTemperature(colorTemperature);
+            }
+        } else {
+            qCWarning(treelandOutput) << "Commit callback received unexpected state pointer!"
+                                      << "Expected:" << newState.get()
+                                      << "Got:" << state.get();
+        }
+    }, WOutputHelper::AfterCommitStage);
+    renderWindow->update(viewport);
+}
