@@ -22,6 +22,7 @@ class SurfaceWrapper : public QQuickItem
     friend class SurfaceContainer;
     friend class SurfaceProxy;
     friend class ShellHandler;
+    friend class LockScreen;
     Q_OBJECT
     QML_ELEMENT
     QML_UNCREATABLE("SurfaceWrapper objects are created by c++")
@@ -78,6 +79,7 @@ public:
         XWayland,
         Layer,
         InputPopup,
+        LockScreen,
     };
     Q_ENUM(Type)
 
@@ -134,6 +136,9 @@ public:
     QRectF geometry() const;
     QRectF normalGeometry() const;
     void moveNormalGeometryInOutput(const QPointF &position);
+    QPointF alignToPixelGrid(const QPointF &pos) const;
+    QRectF alignGeometryToPixelGrid(const QRectF &geometry) const;
+    qreal getOutputDevicePixelRatio(const QPointF &pos) const;
 
     QRectF maximizedGeometry() const;
     void setMaximizedGeometry(const QRectF &newMaximizedGeometry);
