@@ -26,7 +26,7 @@ public:
     bool useLockScreen() const;
     std::optional<QStringList> unescapeExecArgs(const QString &str) noexcept;
     bool tryExec() const;
-    bool disableDebugView() const;
+    bool enableDebugView() const;
 
 private:
     CmdLine();
@@ -38,5 +38,5 @@ private:
     std::unique_ptr<QCommandLineOption> m_run;
     std::unique_ptr<QCommandLineOption> m_lockScreen;
     QCommandLineOption m_tryExec;
-    QCommandLineOption m_disableDebugView;
+    QCommandLineOption m_enableDebugView;
 };
