@@ -1,7 +1,7 @@
 // Copyright (C) 2024 Dingyuan Zhang <lxz@mkacg.com>.
 // SPDX-License-Identifier: Apache-2.0 OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#include "modules/primary-output/outputmanagement.h"
+#include "modules/output-manager/outputmanagement.h"
 
 #include <wserver.h>
 
@@ -29,12 +29,12 @@ private Q_SLOTS:
 
     void testCreate()
     {
-        QVERIFY(m_server->attach<PrimaryOutputV1>(m_server) != nullptr);
+        QVERIFY(m_server->attach<OutputManagerV1>(m_server) != nullptr);
     }
 
     void verifyPrimaryOutput()
     {
-        auto protocol = m_server->findChild<PrimaryOutputV1 *>();
+        auto protocol = m_server->findChild<OutputManagerV1 *>();
         QVERIFY(protocol != nullptr);
     }
 
